@@ -5,12 +5,13 @@
 # Author    : Chris Jones
 # Email     : crj341@student.bham.ac.uk
 # Date      : 26/06/2020
-# Version   : 0.2.0
+# Version   : 0.2.2
 
 import  numpy                   as      np
 import  plotly.graph_objects    as      go
 import  easygui                 as      gui
 from    scipy                   import  interpolate as  interp
+from    NSFopen                 import  read        as  ns
 import  re
 import  os
 
@@ -48,7 +49,10 @@ class AFM:
     retract: [samples, curve_number]
         AFM force curve retract data (nN) produced by the baseline function
         and updated by teh contact function
-    z_position:[samples, curve_number]
+    z_approach:[samples, curve_number]
+        AFM force curve tip/piezo position data (nm) produced by the baseline
+        function and updated by teh contact function
+    z_withdraw:[samples, curve_number]
         AFM force curve tip/piezo position data (nm) produced by the baseline
         function and updated by teh contact function
     adhesion: [curve_number]
@@ -185,7 +189,8 @@ class AFM:
 
         self.approach = []
         self.retract = []
-        self.z_position = []
+        self.z_approach = []
+        self.z_withdraw = []
 
         self.approach_raw = []
         self.retract_raw = []
@@ -266,32 +271,36 @@ class AFM:
         else:
             file = self.file_name
 
-        file_no_ext = file[:-3]
+        if file[-3:] == 'nid':
+            print('input_files function not required for .nid files')
 
-        i = int(file[-3])
-        j = int(file[-2])
-        k = int(file[-1])
+        else:
+            file_no_ext = file[:-3]
 
-        ext = str(i) + str(j) + str(k)
-
-        _filelist = list('')
-
-        while os.path.isfile(file_no_ext + ext):
-            
-            _filelist.append(file_no_ext + ext)
-
-            k += 1
-            if k == 10:
-                k = 0
-                j += 1
-            
-            if j == 10:
-                j = 0
-                i += 1
+            i = int(file[-3])
+            j = int(file[-2])
+            k = int(file[-1])
 
             ext = str(i) + str(j) + str(k)
 
-        self.filelist = _filelist
+            _filelist = list('')
+
+            while os.path.isfile(file_no_ext + ext):
+                
+                _filelist.append(file_no_ext + ext)
+
+                k += 1
+                if k == 10:
+                    k = 0
+                    j += 1
+                
+                if j == 10:
+                    j = 0
+                    i += 1
+
+                ext = str(i) + str(j) + str(k)
+
+            self.filelist = _filelist
 
     def nanoscope_params(
         self,
@@ -436,6 +445,35 @@ class AFM:
 
             i += 1
 
+
+    def nanosurf_import(
+        self
+    ):
+        '''
+        Function to plot import Nanosurf .nid data files.
+        '''
+        _nid = ns.read(filename)
+        _data = _nid.data
+        _params = _nid.param
+
+        _approach = _data.Spec.Forward['Deflection']
+        _z_approach = _data.Spec.Forward['Z-Axis Sensor']
+
+        _withdraw = _data.Spec.Backward['Deflection']
+        _z_withdraw = _data.Spec.Backward['Z-Axis Sensor']
+
+        _def_sens = _params.Tip.Sensitivity['Value']
+        _spr_const = _params.Tip.Prop0['Value']
+
+
+        self.def_sens = float(_def_sens)
+        self.spr_const = float(_spr_const)
+
+        self.approach_raw = _approach
+        self.retract_raw = _withdraw
+        self.z_approach = _z_approach
+        self.z_withdraw = _z_withdraw
+
     def plot_raw(
         self,
         curve_number = 1
@@ -444,14 +482,25 @@ class AFM:
         Function to plot individual, unprocessed force curves. Curve number to
         plot must be given as an input <.plot_raw(n)>
         '''
-        fig = go.Figure()
-        fig.add_trace(go.Scatter(y = self.approach_raw[:,curve_number], x = self.z_position_raw[:,curve_number], mode='lines', name = 'Approach curve (raw)'))
-        fig.add_trace(go.Scatter(y = self.retract_raw[:,curve_number], x = self.z_position_raw[:,curve_number], mode='lines', name = 'Retract curve (raw)'))
-        fig.update_layout(
-            yaxis_title_text='Force (nN)',
-            xaxis_title_text='Piezo position (nm)',
-        )
-        fig.show()
+        if self.file_name[-3:] != 'nid':
+            fig = go.Figure()
+            fig.add_trace(go.Scatter(y = self.approach_raw[:,curve_number], x = self.z_position_raw[:,curve_number], mode='lines', name = 'Approach curve (raw)'))
+            fig.add_trace(go.Scatter(y = self.retract_raw[:,curve_number], x = self.z_position_raw[:,curve_number], mode='lines', name = 'Retract curve (raw)'))
+            fig.update_layout(
+                yaxis_title_text='Force (nN)',
+                xaxis_title_text='Piezo position (nm)',
+            )
+            fig.show()
+
+        else:
+            fig = go.Figure()
+            fig.add_trace(go.Scatter(y = self.approach_raw[:,curve_number], x = self.z_approach[:,curve_number], mode='lines', name = 'Approach curve (raw)'))
+            fig.add_trace(go.Scatter(y = self.retract_raw[:,curve_number], x = self.z_withdraw[:,curve_number], mode='lines', name = 'Retract curve (raw)'))
+            fig.update_layout(
+                yaxis_title_text='Force (nN)',
+                xaxis_title_text='Piezo position (nm)',
+            )
+            fig.show()
 
     def baseline(
         self,
@@ -473,7 +522,8 @@ class AFM:
         '''
         self.approach = np.zeros([self.samps_line, len(self.filelist)])
         self.retract = np.zeros([self.samps_line, len(self.filelist)])
-        self.z_position = np.zeros([self.samps_line * 2, len(self.filelist)])
+        self.z_approach = np.zeros([self.samps_line, len(self.filelist)])
+        self.z_withdraw = np.zeros([self.samps_line, len(self.filelist)])
 
         i = 0
         
@@ -481,15 +531,13 @@ class AFM:
 
             _approach = self.approach_raw[:,curve_number]
             _retract = self.retract_raw[:,curve_number]
-            _z = self.z_position_raw[:,curve_number]
 
-            baseline_start = int(len(_z) * start_pos)
-            baseline_end = int(len(_z) * end_pos)
+            baseline_start = int(len(_approach) * start_pos)
+            baseline_end = int(len(_approach) * end_pos)
             baseline_region = np.arange(baseline_start,baseline_end)
 
             _approach_region = _approach[baseline_region]
             _retract_region = _retract[baseline_region]
-            _z_region = _z[baseline_region]
 
             _approach_mean = np.mean(_approach_region)
 
@@ -499,18 +547,26 @@ class AFM:
             approach_dev = np.std(_approach_region)
             retract_dev = np.std(_retract_region)
 
-            piezo_position = np.concatenate((_z, _z), axis = 0)
+            if self.file_name[-3:] != 'nid':
+                _z = self.z_position_raw[:,curve_number]
+                piezo_position = np.concatenate((_z, _z), axis = 0)
 
             if approach_dev < max_approach_noise and retract_dev < max_retract_noise:
                 self.approach[:,i] = adjusted_approach
                 self.retract[:,i] = adjusted_retract
-                self.z_position[:,i] = piezo_position
+
+                if self.file_name[-3:] != 'nid':
+                    self.z_approach[:,i] = piezo_position
+                    self.z_withdraw[:,i] = piezo_position
 
                 i += 1
 
         self.approach = self.approach[:,:i]
         self.retract = self.retract[:,:i]
-        self.z_position = self.z_position[:,:i]
+
+        if self.file_name[-3:] != 'nid':
+            self.z_approach = self.z_approach[:,:i]
+            self.z_withdraw = self.z_withdraw[:,:i]
 
     def contact(self):
         '''
@@ -519,10 +575,10 @@ class AFM:
         and the previous value. Note this method may be unstable for noisy curves.
         '''
         
-        for i in range(0, self.z_position.shape[1]):
+        for i in range(0, self.z_approach.shape[1]):
             _approach = self.approach[:,i]
             _retract = self.retract[:,i]
-            _z = self.z_position[:,i]
+            _z = self.z_approach[:,i]
 
             try:
                 j = 0
@@ -536,7 +592,8 @@ class AFM:
                 f = interp.interp1d(x,y)
                 intercept = f(0)
 
-                self.z_position[:,i] = self.z_position[:,i] - intercept
+                self.z_approach[:,i] = self.z_approach[:,i] - intercept
+                self.z_withdraw[:,i] = self.z_withdraw[:,i] - intercept
 
             except:
                 print('WARNING: Could not locate contact point for curve number ', i, '. Consider checking baseline alignment.')
@@ -545,18 +602,18 @@ class AFM:
         self
     ):
 
-        for i in range(0, self.z_position.shape[1]):
+        for i in range(0, self.z_approach.shape[1]):
 
-            _z = self.z_position[:,i]
+            _z_approach = self.z_approach[:,i]
+            _z_withdraw = self.z_withdraw[:,i]
             _approach = self.approach[:,i]
             _retract = self.retract[:,i]
                 
-            tip_approach = _z[:self.samps_line] + _approach / self.spr_const
-            tip_retract = _z[self.samps_line:] + _retract / self.spr_const
+            tip_approach = _z_approach + _approach / self.spr_const
+            tip_retract = _z_withdraw + _retract / self.spr_const
 
-            tip_position = np.concatenate((tip_approach, tip_retract), axis = 0)
-
-            self.z_position[:,i] = tip_position
+            self.z_approach[:,i] = tip_approach
+            self.z_withdraw[:,i] = tip_retract
 
     def plot_adjusted(
         self,
@@ -567,8 +624,8 @@ class AFM:
         plot must be given as an input <.plot_adjusted(n)>
         '''
         fig = go.Figure()
-        fig.add_trace(go.Scatter(y = self.approach[:,curve_number], x = self.z_position[:,curve_number][:self.samps_line], mode='lines', name = 'Approach curve'))
-        fig.add_trace(go.Scatter(y = self.retract[:,curve_number], x = self.z_position[:,curve_number][self.samps_line:], mode='lines', name = 'Retract curve'))
+        fig.add_trace(go.Scatter(y = self.approach[:,curve_number], x = self.z_approach[:,curve_number], mode='lines', name = 'Approach curve'))
+        fig.add_trace(go.Scatter(y = self.retract[:,curve_number], x = self.z_withdraw[:,curve_number], mode='lines', name = 'Retract curve'))
         fig.update_layout(
             yaxis_title_text='Force (nN)',
             xaxis_title_text='Distance, z (nm)',
@@ -583,20 +640,20 @@ class AFM:
         trace_1 = []
         trace_2 = []
 
-        for step in range(0, self.z_position.shape[1]):
+        for step in range(0, self.z_approach.shape[1]):
 
             trace_1.append(go.Scatter(
                         visible=False,
                         name= 'Approach ' + str(step),
                         y = self.approach[:, step],
-                        x = self.z_position[:, step][:self.samps_line]
+                        x = self.z_approach[:, step]
                     ))
 
             trace_2.append(go.Scatter(
                         visible=False,
                         name= 'Retract ' + str(step),
                         y = self.retract[:, step],
-                        x = self.z_position[:, step][self.samps_line:]
+                        x = self.z_withdraw[:, step]
                     ))
 
         fig = go.Figure(data = trace_1 + trace_2)
@@ -650,7 +707,8 @@ class AFM:
 
             self.approach = np.delete(self.approach, curve_int, axis = 1)
             self.retract = np.delete(self.retract, curve_int, axis = 1)
-            self.z_position = np.delete(self.z_position, curve_int, axis = 1)
+            self.z_approach = np.delete(self.z_approach, curve_int, axis = 1)
+            self.z_withdraw = np.delete(self.z_withdraw, curve_int, axis = 1)
 
             try:
                 self.adhesion = np.delete(self.adhesion, curve_int, axis = 1)
@@ -685,7 +743,8 @@ class AFM:
             output_path,
             approach = self.approach,
             retract = self.retract,
-            z_position = self.z_position,
+            z_approach = self.z_approach,
+            z_withdraw = self.z_withdraw,
             adhesion = self.adhesion,
             modulus = self.modulus,
             data_offset = self.data_offset,
@@ -718,7 +777,8 @@ class AFM:
 
         self.approach = data['approach']
         self.retract = data['retract']
-        self.z_position = data['z_position']
+        self.z_approach = data['z_approach']
+        self.z_withdraw = data['z_withdraw']
         self.adhesion = data['adhesion']
         self.modulus = data['modulus']
         self.data_offset = data['data_offset']
@@ -742,9 +802,9 @@ class AFM:
         setting the 'plot_hist' input to 'True'
         '''
 
-        self.adhesion = np.zeros(self.z_position.shape[1])
+        self.adhesion = np.zeros(self.z_approach.shape[1])
 
-        for curve_number in range(0, self.z_position.shape[1]):
+        for curve_number in range(0, self.z_approach.shape[1]):
             self.adhesion[curve_number] = np.amin(self.retract[:, curve_number]) * -1
 
         if plot_hist is True:
@@ -878,16 +938,16 @@ class AFM:
         A histogram of calculated values can be plotted by setting the 'plot_hist'
         input to 'True'
         '''
-        self.modulus = np.zeros(self.z_position.shape[1])
+        self.modulus = np.zeros(self.z_approach.shape[1])
 
         if soft_indenter is True:
             indenter_term = (1 - indenter_poisson_ratio ** 2) / (indenter_modulus * 1e6)
         else:
             indenter_term = 0
 
-        for curve_number in range(0, self.z_position.shape[1]):
+        for curve_number in range(0, self.z_approach.shape[1]):
             _approach = self.approach[:,curve_number]
-            _z = self.z_position[:,curve_number]
+            _z = self.z_approach[:,curve_number]
 
             force = _approach[0] * 1e-9
             depth = _z[0] * 1e-9
